﻿/////////////////////////////////////////////////////////////////////////////
//	Psytec QR Code Decode Library [Ver.1.60 2024/12/24]
//  PsyQrDcd.h
//	DLL インポート用ヘッダファイル
//	(C) 2003-2024 Psytec Inc.
//	URL:https://www.psytec.co.jp/
/////////////////////////////////////////////////////////////////////////////

#if !defined(PSYQRDCD_H)
#define PSYQRDCD_H

// リンクライブラリの指定
#ifdef _WIN64
	#pragma comment(lib, ".\\lib\\x64\\PsyQrDcd.lib")
#else
	#pragma comment(lib, ".\\lib\\x86\\PsyQrDcd.lib")
#endif


// エラーコード定義
#define QRD_ERROR_SYMBLE_NOT_FOUND	 0
#define QRD_ERROR_FILE_NOT_FOUND	-1
#define QRD_ERROR_READ_FAULT		-2
#define QRD_ERROR_BAD_FORMAT		-3
#define QRD_ERROR_SHARING_VIOLATION	-4
#define QRD_ERROR_NOT_ENOUGH_MEMORY	-5


#ifdef _UNICODE
	#define DecodePictureFile		DecodePictureFileW
	#define DecodePictureFileRect	DecodePictureFileRectW
	#define GetDecodeDataString		GetDecodeDataStringW
#else
	#define DecodePictureFile		DecodePictureFileA
	#define DecodePictureFileRect	DecodePictureFileRectA
	#define GetDecodeDataString		GetDecodeDataStringA
#endif

// DLL ファンクション
extern "C"
{
/////////////////////////////////////////////////////////////////////////////
// DecodePictureFile - 画像ファイルのデコード
// 引  数：画像ファイルパス(BMP、JPEG、GIF、PING、TIFF形式)
// 戻り値：デコードされたシンボル数、または上記エラーコード

__declspec(dllimport) int __stdcall DecodePictureFileA(LPCSTR lpszFileName);
__declspec(dllimport) int __stdcall DecodePictureFileW(LPCWSTR lpszFileName);


/////////////////////////////////////////////////////////////////////////////
// DecodePictureFileRect - 画像ファイルのデコード（デコード範囲指定）
// 引  数：画像ファイルパス(BMP、JPEG、GIF、PING、TIFF形式)、デコード範囲座標（左ｘ、上ｙ、右ｘ、下ｙ）
// 戻り値：デコードされたシンボル数、または上記エラーコード
// 備  考：座標は画像全体の左上を原点としたピクセル単位で指定してください。

__declspec(dllimport) int __stdcall DecodePictureFileRectA(LPCSTR lpszFileName, int nLeftX, int nTopY, int nRightX, int nBottomY);
__declspec(dllimport) int __stdcall DecodePictureFileRectW(LPCWSTR lpszFileName, int nLeftX, int nTopY, int nRightX, int nBottomY);


/////////////////////////////////////////////////////////////////////////////
// DecodeGrayScaleArray - グレースケール(256階調)データのデコード
// 引  数：グレースケールデータ配列の画像データ(BYTE型)、画像幅、画像高さ
// 戻り値：デコードされたシンボル数、または QRD_ERROR_SYMBLE_NOT_FOUND, QRD_ERROR_NOT_ENOUGH_MEMORY
// 備  考：画像データは左上より横方向、１ピクセルにつき１バイト、256階調、0=黒､ 255=白

__declspec(dllimport) int __stdcall DecodeGrayScaleArray(LPBYTE lpbyData, int nWidth, int nHeight);


/////////////////////////////////////////////////////////////////////////////
// DecodeBinaryArray - バイナリ(２値)データのデコード
// 引  数：バイナリデータ配列の画像データ(BYTE型)、画像幅、画像高さ
// 戻り値：デコードされたシンボル数、または QRD_ERROR_SYMBLE_NOT_FOUND, QRD_ERROR_NOT_ENOUGH_MEMORY
// 備  考：画像データは左上より横方向、１ピクセルにつき１バイト、２値、明=FALSE(0)､ 暗=TRUE(0以外)

__declspec(dllimport) int __stdcall DecodeBinaryArray(LPBYTE lpbyData, int nWidth, int nHeight);


/////////////////////////////////////////////////////////////////////////////
// GetDecodeData - デコード結果データ取得(バイト配列)
// 引  数：データインデックス(0基数)、データ受け取りバッファアドレス
// 戻り値：データ長(バイト数)、-1=データインデックス不正
// 備  考：データ終端には'\0'が付加されています。(戻り値の長さには'\0'は含まれていません。)
//		   呼び出し時 lpbyBuffの値により動作が異なります。
//		   =NULL → 内部でバッファリングしているデータのアドレスをlpbyBuffに代入します。
//		   !=NULL → 指定されたアドレスにバッファリングしているデータをコピーします。

__declspec(dllimport) int __stdcall GetDecodeData(int nIndex, LPBYTE lpbyBuff[]);


/////////////////////////////////////////////////////////////////////////////
// GetDecodeDataStringW - デコード結果データ取得(WCHAR文字列)
// 引  数：データインデックス(0基数)、データ受け取りバッファアドレス、UTF-8読み取り優先フラグ
// 戻り値：データ長(バイト数)、-1=データインデックス不正
// 備  考：データ終端には'\0'が付加されています。(戻り値の長さには'\0'は含まれていません。)
//		   呼び出し時 lpwszBuffの値により動作が異なります。
//		   =NULL → 内部でバッファリングしているデータのアドレスをlpwszBuffに代入します。
//		   !=NULL → 指定されたアドレスにバッファリングしているデータをコピーします。

__declspec(dllimport) int __stdcall GetDecodeDataStringW(int nIndex, LPWSTR lpwszBuff[], BOOL bPrioritizeUTF8 = FALSE);


/////////////////////////////////////////////////////////////////////////////
// GetDecodeDataStringA - デコード結果データ取得(CHAR文字列)
// 引  数：データインデックス(0基数)、データ受け取りバッファアドレス、UTF-8読み取り優先フラグ
// 戻り値：データ長(バイト数)、-1=データインデックス不正
// 備  考：データ終端には'\0'が付加されています。(戻り値の長さには'\0'は含まれていません。)
//		   呼び出し時 lpszBuffの値により動作が異なります。
//		   =NULL → 内部でバッファリングしているデータのアドレスをlpszBuffに代入します。
//		   !=NULL → 指定されたアドレスにバッファリングしているデータをコピーします。

__declspec(dllimport) int __stdcall GetDecodeDataStringA(int nIndex, LPSTR lpszBuff[], BOOL bPrioritizeUTF8 = FALSE);

/*
《UTF-8読み取り優先フラグについて 》
 
QRコードシンボルの文字列データは、元々は日本語のシフトJIS文字コードを前提としておりましたが、海外での利用や絵文字の普及などもあって
UTF-8が使用される機会も増えています。

一般的な日本語の文字列であれば文字コードの規則からシフトJISとUTF-8は比較的容易に判別できるのですが、一部の文字や文字列パターンでは
どちらにでも解釈できる場合があります。

例えばUTF-8で'α'は2バイトで[CE][B1]ですが、シフトJISでは半角カタカナが有効なため'ﾎｱ'と読むことができます。
この場合、UTF-8読み取り優先フラグをTRUEにセットすると取得できる文字列は'α'となり、FALSEにセットすると'ﾎｱ'となります。

シフトJISかUTF-8かが判別できたときはフラグは無視され、設定に関係なく同じ文字列が取得されます。
なお、ASCII文字列(半角の英数字や記号)はシフトJISもUTF-8も同じ1バイトの文字コードですので影響はありません。
*/


/////////////////////////////////////////////////////////////////////////////
// GetDecodeVersion - デコードシンボルの型番取得
// 引  数：データインデックス(0基数)
// 戻り値：型番(1～40)、-1=データインデックス不正

__declspec(dllimport) int __stdcall GetDecodeVersion(int nIndex);


/////////////////////////////////////////////////////////////////////////////
// GetDecodeLevel - デコードシンボルの誤り訂正レベル取得
// 引  数：データインデックス(0基数)
// 戻り値：誤り訂正レベル(0=L, 1=M, 2=Q, 3=H)、-1=データインデックス不正

__declspec(dllimport) int __stdcall GetDecodeLevel(int nIndex);


/////////////////////////////////////////////////////////////////////////////
// GetDecodeMaskingNo - デコードシンボルのマスキングパターン番号取得
// 引  数：データインデックス(0基数)
// 戻り値：マスキングパターン番号(0～7)、-1=データインデックス不正

__declspec(dllimport) int __stdcall GetDecodeMaskingNo(int nIndex);


/////////////////////////////////////////////////////////////////////////////
// GetConcatenationInfo - デコードシンボルの連結情報取得
// 引  数：データインデックス(0基数)、連結シーケンス(ポインタ)、連結チェックデジット(ポインタ)
// 戻り値：データ連結数(1～16)、0=非連結データ、-1=データインデックス不正

__declspec(dllimport) int __stdcall GetConcatenationInfo(int nIndex, LPINT lpnSeqNo, LPBYTE lpbyCheckDigit);


/////////////////////////////////////////////////////////////////////////////
// GetSymbolePosition - デコードシンボルの位置情報取得
// 引  数：データインデックス(0基数)、座標受け取り配列
// 戻り値：0=正常終了、-1=データインデックス不正
// 備  考：座標受け取り配列は、正立状態のシンボルの左上から時計回り
//		   [0]=左上ｘ、[1]=左上ｙ、[2]=右上ｘ、[3]=右上ｙ、[4]=右下ｘ、[5]=右下ｙ、[6]=左下ｘ、[7]=左下ｙ
//		   POINT[4]をキャストして渡すことも可能です。

__declspec(dllimport) int __stdcall GetSymbolePosition(int nIndex, int nPositon[8]);


/////////////////////////////////////////////////////////////////////////////
// SetDecodeSymbolCount - デコードシンボル数の上限設定
// 引  数：デコードシンボル上限数(1～32)、0=読み取り成功時点のレベルで処理を終了(初期値)
// 備  考：デコード処理内では様々な画像に対応するため複数の画像加工処理を行っています。
//		   0を設定した場合、同一レベルの処理で読み取ることができるシンボルは全て読み取ります。
//		   1以上の設定では上限数に達すると処理を終えますが、逆に上限数に達するまで処理を終えませんので、
//		   読み落としを防ぎたい場合にもご利用ください。
//		   設定値はプログラム終了(DLLアンロード)まで有効です。

__declspec(dllimport) void __stdcall SetDecodeSymbolCount(int nCount);


/////////////////////////////////////////////////////////////////////////////
// SetDecodeEffectLevel - 画像処理レベルの設定
// 引  数：デコードレベル(0～3)、初期値=3
// 備  考：0=最も単純な２値化処理のみ行います。
//		   1=設定0に加えノイズ除去処理を行います。
//		   2=設定0に加え画像の部分的な明暗補正処理をを行います。
//		   3=設定1と設定2の両方の処理を行います。
//		   設定値はプログラム終了(DLLアンロード)まで有効です。

__declspec(dllimport) void __stdcall SetDecodeEffectLevel(int nLevel);


/////////////////////////////////////////////////////////////////////////////
// SetBrightnessBorder - 画像明暗しきい値設定
// 引  数：明暗しきい値(0～255)、0=自動設定(初期値)
// 備  考：自動設定時には画像全体で最も明るい点と暗い点の中間値をしきい値としています。
//		   シンボル以外の画像が含まれていて、しきい値が正しく求められない場合などに設定してください。
//		   設定値はプログラム終了(DLLアンロード)まで有効です。

__declspec(dllimport) void __stdcall SetBrightnessBorder(BYTE byBorder);


/////////////////////////////////////////////////////////////////////////////
// SetMinimumModuleSize - シンボル最小モジュール単位設定
// 引  数：モジュールサイズ(1～255)、初期値=1
// 備  考：モジュールの大きなシンボルを認識する場合、適切な値を設定することで処理を高速化できます。
//		   設定値はプログラム終了(DLLアンロード)まで有効です。

__declspec(dllimport) void __stdcall SetMinimumModuleSize(BYTE bySize);


/////////////////////////////////////////////////////////////////////////////
// FreeAllocateMemory - デコード結果保存メモリの開放
// 備  考：デコード結果のデータやシンボル情報を保存しているメモリを開放します。
//		   メモリの解放はデコード指示関数の呼び出し時やDLLアンロード時にも自動的に行われます。

__declspec(dllimport) void __stdcall FreeAllocateMemory();


/////////////////////////////////////////////////////////////////////////////
// GetErrorLevel - 読み取りエラー詳細情報取得
// 戻り値：0=エラーなし、1=位置検出パターン未検出、2=位置検出パターンからのシンボル検出不可、3=データ部読み取り失敗

__declspec(dllimport) int __stdcall GetErrorLevel();

}

#endif // !defined(PSYQRDCD_H)
