﻿
// SampleDlg.cpp : 実装ファイル
//

#include "pch.h"
#include "framework.h"
#include "Sample.h"
#include "SampleDlg.h"
#include "afxdialogex.h"

#include "SetRectDlg.h"
#include "SetPropertyDlg.h"

#include "PsyQrDcd.h" // QR Code Decode Library ヘッダーファイル

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSampleDlg ダイアログ


CSampleDlg::CSampleDlg(CWnd* pParent /*=nullptr*/)
	: CDialogEx(IDD_SAMPLE_DIALOG, pParent)
{
	m_pMenu = NULL;

	m_bEnableSetRect = FALSE;
	m_nRectLeft	  = 0;
	m_nRectTop	  = 0;
	m_nRectRight  = 0;
	m_nRectBottom = 0;

	m_nSymbolCount = 0;
	m_nEffectLevel = 3;
	m_byBorder     = 0;
	m_byModuleSize = 1;

	m_bPrioritizeUTF8 = FALSE;

	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CSampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CSampleDlg, CDialogEx)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_SIZE()
	ON_WM_CTLCOLOR()
	ON_WM_DESTROY()
	ON_WM_DROPFILES()
	ON_COMMAND(ID_FILE_OPEN, &CSampleDlg::OnFileOpen)
	ON_COMMAND(ID_SET_DECODE_RECT, &CSampleDlg::OnSetDecodeRect)
	ON_COMMAND(ID_SET_DECODE_PROPERTY, &CSampleDlg::OnSetDecodeProperty)
	ON_COMMAND(ID_PRIORITIZE_UTF8, &CSampleDlg::OnPrioritizeUTF8)
END_MESSAGE_MAP()


// CSampleDlg メッセージ ハンドラー

BOOL CSampleDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// このダイアログのアイコンを設定します。アプリケーションのメイン ウィンドウがダイアログでない場合、
	//  Framework は、この設定を自動的に行います。
	SetIcon(m_hIcon, TRUE);			// 大きいアイコンの設定
	SetIcon(m_hIcon, FALSE);		// 小さいアイコンの設定

	// メニュー設定
	m_pMenu = new CMenu;
	m_pMenu->LoadMenu(IDR_DLGMENU);
	SetMenu(m_pMenu);

	m_pMenu->CheckMenuItem(ID_PRIORITIZE_UTF8, MF_BYCOMMAND | (m_bPrioritizeUTF8 ? MF_CHECKED : MF_UNCHECKED));

	return TRUE;  // フォーカスをコントロールに設定した場合を除き、TRUE を返します。
}

// ダイアログに最小化ボタンを追加する場合、アイコンを描画するための
//  下のコードが必要です。ドキュメント/ビュー モデルを使う MFC アプリケーションの場合、
//  これは、Framework によって自動的に設定されます。

void CSampleDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // 描画のデバイス コンテキスト

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// クライアントの四角形領域内の中央
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// アイコンの描画
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialogEx::OnPaint();
	}
}

// ユーザーが最小化したウィンドウをドラッグしているときに表示するカーソルを取得するために、
//  システムがこの関数を呼び出します。
HCURSOR CSampleDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


void CSampleDlg::OnDestroy()
{
	CDialogEx::OnDestroy();

	// メニュー開放
	delete m_pMenu;
}

void CSampleDlg::OnSize(UINT nType, int cx, int cy)
{
	CDialogEx::OnSize(nType, cx, cy);

	// エディットコントロールををクライアント領域に合わせて表示
	CWnd* pWndEdit = GetDlgItem(IDC_DECODE_RESULT);

	if (pWndEdit != NULL)
	{
		RECT rc;
		GetClientRect(&rc);

		pWndEdit->MoveWindow(&rc);
	}
}

HBRUSH CSampleDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	// テキストボックス背景を白に変更
	return (pWnd->GetDlgCtrlID() == IDC_DECODE_RESULT) ? (HBRUSH)GetStockObject(WHITE_BRUSH) : CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
}

void CSampleDlg::OnDropFiles(HDROP hDropInfo)
{
	TCHAR szFilePath[MAX_PATH]; // ドロップファイルパス

	// ドロップファイルパス取得
	DragQueryFile(hDropInfo, 0, szFilePath, MAX_PATH - 1);
	DragFinish(hDropInfo);

	DecodeFile(szFilePath);
}

void CSampleDlg::OnFileOpen()
{
	static int nLoadFilterIndex = 1;

	// CFileDialog クラス構築
	CFileDialog* pFileDialog = new CFileDialog(TRUE, NULL, NULL, OFN_HIDEREADONLY | OFN_PATHMUSTEXIST,
											   _T("ピクチャ ファイル (*.bmp;*.jpg;*.jpeg;*.gif;*.tif;*.tiff;*.png)|*.bmp;*.jpg;*.jpeg;*.gif;*.tif;*.tiff;*.png|")
											   _T("ビットマップ ファイル (*.bmp)|*.bmp|")
											   _T("JPEG 形式 (*.jpg;*.jpeg)|*.jpg;*.jpeg|")
											   _T("GIF 形式 (*.gif)|*.gif|")
											   _T("TIFF 形式 (*.tif;*.tiff)|*.tif;*.tiff|")
											   _T("PNG 形式 (*.png)|*.png|")
											   _T("全てのファイル (*.*)|*.*||"), this);
	pFileDialog->m_ofn.lpstrTitle = _T("開く");
	pFileDialog->m_ofn.nFilterIndex	= nLoadFilterIndex;

	CString strPathName;

	//「ファイルオープン」ダイアログ表示
	if (pFileDialog->DoModal() == IDOK)
	{
		strPathName = pFileDialog->GetPathName();
		nLoadFilterIndex = pFileDialog->m_ofn.nFilterIndex;
	}

	delete 	pFileDialog;

	if (strPathName.IsEmpty())
		return;

	DecodeFile(strPathName);
}

void CSampleDlg::OnSetDecodeRect()
{
	//「デコード範囲指定」設定画面表示

	CSetRectDlg* pSetRectDlg = new CSetRectDlg(this);

	pSetRectDlg->m_bEnableSetRect = m_bEnableSetRect;
	pSetRectDlg->m_nRectLeft   = m_nRectLeft;
	pSetRectDlg->m_nRectTop	   = m_nRectTop;
	pSetRectDlg->m_nRectRight  = m_nRectRight;
	pSetRectDlg->m_nRectBottom = m_nRectBottom;

	if (pSetRectDlg->DoModal() == IDOK)
	{
		m_bEnableSetRect = pSetRectDlg->m_bEnableSetRect;
		m_nRectLeft   = pSetRectDlg->m_nRectLeft;
		m_nRectTop    = pSetRectDlg->m_nRectTop;
		m_nRectRight  = pSetRectDlg->m_nRectRight;
		m_nRectBottom = pSetRectDlg->m_nRectBottom;
	}

	delete pSetRectDlg;
}

void CSampleDlg::OnSetDecodeProperty()
{
	//「デコード条件設定」設定画面表示

	CSetPropertyDlg* pSetPropertyDlg = new CSetPropertyDlg(this);

	pSetPropertyDlg->m_nSymbolCount = m_nSymbolCount;
	pSetPropertyDlg->m_nEffectLevel = m_nEffectLevel;
	pSetPropertyDlg->m_byBorder	 = m_byBorder;
	pSetPropertyDlg->m_byModuleSize = m_byModuleSize;

	if (pSetPropertyDlg->DoModal() == IDOK)
	{
		m_nSymbolCount = pSetPropertyDlg->m_nSymbolCount;
		m_nEffectLevel = pSetPropertyDlg->m_nEffectLevel;
		m_byBorder     = pSetPropertyDlg->m_byBorder;
		m_byModuleSize = pSetPropertyDlg->m_byModuleSize;

		SetDecodeSymbolCount(m_nSymbolCount);
		SetDecodeEffectLevel(m_nEffectLevel);
		SetBrightnessBorder(m_byBorder);
		SetMinimumModuleSize(m_byModuleSize);
	}

	delete pSetPropertyDlg;

}

void CSampleDlg::OnPrioritizeUTF8()
{
	// UTF-8優先フラグ切り替え

	m_bPrioritizeUTF8 = !m_bPrioritizeUTF8;

	m_pMenu->CheckMenuItem(ID_PRIORITIZE_UTF8, MF_BYCOMMAND | (m_bPrioritizeUTF8 ? MF_CHECKED : MF_UNCHECKED));
}

void CSampleDlg::DecodeFile(LPCTSTR lpszPathName)
{
	// ファイル指定によるデコード

	static LPCTSTR lpcszSeparator = _T("───────────────────────────\r\n");
	static LPCTSTR lpcszLevel[] = {_T("L(7%)"), _T("M(15%)"), _T("Q(25%)"), _T("H(30%)")};

	// 砂時計カーソル表示
	CWaitCursor wc;

	CString strResult; // デコード結果文字列（エディットコントロール表示用）

	int nResult;

	if (! m_bEnableSetRect)
		nResult = DecodePictureFile(lpszPathName); // 範囲指定なし
	else
		nResult = DecodePictureFileRect(lpszPathName, m_nRectLeft, m_nRectTop, m_nRectRight, m_nRectBottom); // 範囲指定あり

	if (nResult >= 1)
	{
		// デコード正常終了
		for (int i = 0; i < nResult; ++i)
		{
			strResult += lpcszSeparator;

			CString strWork;

			// シンボル型番、誤り訂正レベル
			strWork.Format(_T("デコード データ %d [型番:%d, レベル:%s"), i + 1, GetDecodeVersion(i), lpcszLevel[GetDecodeLevel(i)]);
			strResult += strWork;

			// シンボル連結情報
			int nSeqNo;
			BYTE byCheckDigit;

			int ncConcatenation = GetConcatenationInfo(i, &nSeqNo, &byCheckDigit);

			if (ncConcatenation > 0)
			{
				strWork.Format(_T(", 連結:%d/%d(CD:%02XH)"), nSeqNo + 1, ncConcatenation, byCheckDigit);
				strResult += strWork;
			}

			strResult += "]\r\n";

			strResult += lpcszSeparator;

			// シンボルデータ
			LPTSTR lpszData = NULL;

			GetDecodeDataString(i, &lpszData, m_bPrioritizeUTF8);

			strResult += (LPTSTR)lpszData;

			strResult += "\r\n\r\n";

			// シンボル位置情報
			POINT pointPosition[4];

			GetSymbolePosition(i, (LPINT)pointPosition);

			strWork.Format(_T("シンボル位置：(%d, %d)-(%d, %d)-(%d, %d)-(%d, %d)"),
						   pointPosition[0].x, pointPosition[0].y, pointPosition[1].x, pointPosition[1].y, 
						   pointPosition[2].x, pointPosition[2].y, pointPosition[3].x, pointPosition[3].y); 
			strResult += strWork;
			strResult += "\r\n\r\n";
		}

		// デコード結果保存メモリを開放
		FreeAllocateMemory();
	}
	else
	{
		switch (nResult)
		{
		case QRD_ERROR_SYMBLE_NOT_FOUND:
			strResult = _T("QRコード シンボルを読み取れませんでした。");

			switch (GetErrorLevel())
			{
			case 1:
				strResult += _T("\r\n(位置検出パターンが見つかりませんでした。)");
				break;

			case 2:
				strResult += _T("\r\n(位置検出パターンからシンボルを認識できませんでした。)");
				break;

			case 3:
				strResult += _T("\r\n(データ部の読み取りに失敗しました。)");
				break;

			default:
				break;
			}
			break;

		case QRD_ERROR_FILE_NOT_FOUND:
			strResult = _T("ファイルが見つかりませんでした。");
			break;

		case QRD_ERROR_READ_FAULT:
			strResult = _T("ファイル読み込み中にエラーが発生しました。");
			break;

		case QRD_ERROR_BAD_FORMAT:
			strResult = _T("読み取りできないファイル種別です。");
			break;

		case QRD_ERROR_SHARING_VIOLATION:
			strResult = _T("共有違反によりファイルを読み取ることができませんでした。");
			break;

		case QRD_ERROR_NOT_ENOUGH_MEMORY:
			strResult = _T("メモリ不足です。");
			break;

		default:
			break;
		}
	}

	SetDlgItemText(IDC_DECODE_RESULT, strResult);
}
